import time, sys
FanString = ["PWM (4-pol.)","Voltage (3-pol.)"]

def clamp(val, min, max):
        if val < min: return min
        if val > max: return max
        return val

def skal (x, x1, x2, y1, y2):
       if x > x2: return y2
       if x < x1: return y1
       m = (y2 - y1) / (x2 - x1)
       y = m * x + y1
       return y

def FanParameter():
     global FanOK
     global maxRPM
     global minPWM
     global FanType
     TYPE_PWM = 0
     TYPE_VOLTAGE = 1

     # teste und speichere die maximal einstellbare Luefterdrehzahl 
     open("/proc/stb/fp/fan_vlt", "w").write("ff")
     open("/proc/stb/fp/fan_pwm", "w").write("ff")
     time.sleep (5)
     maxRPM = int(open("/proc/stb/fp/fan_speed").read().split(' ')[0])

     # Test ob Luefter eingebaut - Drehzahl darf nicht mehr 0 sein
     # sonst Tacho defekt oder kein Luefter eingebaut 
     if maxRPM == 0:
          FanOK = False
          open("/proc/stb/fp/fan_vlt", "w").write("00")
          open("/proc/stb/fp/fan_pwm", "w").write("00")
          return
     else:
         FanOK = True

     # was fuer einen Luefter haben wir denn eingebaut
     # reagiert er auf fan_pwm dann pwm-Luefter (4pol), sonst Spannung (3pol)
     open("/proc/stb/fp/fan_pwm", "w").write("00")
     time.sleep (5)
     minPWM = int(open("/proc/stb/fp/fan_speed").read().split(' ')[0])
     if minPWM < maxRPM * 0.8:
          FanType = TYPE_PWM
     else:
          FanType = TYPE_VOLTAGE
     return

class TemperatureSensor(object):
        pass

class FPTemperature(TemperatureSensor):
        def __init__(self, index):
                self.index = index

        def readTemp(self):
                t = int(open("/proc/stb/fp/temp%d"%self.index).read()) / 256.0 * 3.3
                t = 4.246e1 * t + 7.362e-1
                return t

        current = property(readTemp)

class FanControl(object):
        pass

class FPFanControl(FanControl):
        TYPE_PWM = 0
        TYPE_VOLTAGE = 1
         
        def __init__(self, type):
                self.zeroRPM = False
                self.type = type
                # in case of a PWM fan, set voltage to max
                if type == self.TYPE_PWM:
                        open("/proc/stb/fp/fan_vlt", "w").write("ff")
                else:
                        # and vice-versa, but that doesn't really matter.
                        open("/proc/stb/fp/fan_pwm", "w").write("ff")

        def setOutput(self, output):
                print "set fan output to %d" % output,
                if self.type == self.TYPE_PWM:
                     open("/proc/stb/fp/fan_pwm", "w").write("%x" % output)
                     print "pwm"
                else:
                     open("/proc/stb/fp/fan_vlt", "w").write("%x" % output)
                     print "vlt"

        output = property(None, setOutput)
        
        def getRPM(self):
                return int(open("/proc/stb/fp/fan_speed").read().split(' ')[0])

        rpm = property(getRPM)

class FanLoop(object):
        pass

class PID:
        def __init__(self, Kp = 1/100.0, Ki=1/100.0, Kd=0):
                self.Kp = Kp
                self.Ki = Ki
                self.Kd = Kd
                self.i = 0
                self.prev = 0

        def cycle(self, current, target, dT = 1.0):
                error = target - current
                self.i += error * dT
                
                deriv = (error - self.prev) / dT
                out = self.Kp * error + self.Ki * self.i + self.Kd * deriv
                self.prev = error
                return out

class FanLoopPID(FanLoop):

        def __init__(self, fan):
                self.fan = fan
                self.target_rpm = 1000
                self.pid = PID()

        def cycle(self, dT=1.0):
                cur_rpm = self.fan.rpm
                out = self.pid.cycle(cur_rpm, self.target_rpm)
                print "current rpm: %d, target_rpm: %d" % (cur_rpm, self.target_rpm),
                self.fan.output = clamp(out, 0, 255)

class TemperatureLoop:
        def __init__(self, sensors):
                self.FAN_MIN     = 1000
                self.FAN_MAX     = 9000
                self.target_temp = 50
                self.Range       = 5
                self.maxTemp     = 55.0
                self.sensors     = sensors
                self.Alert       = False
                self.noLED       = False
                self.noALERT     = False
        def readCurrentTemperature(self):
                return max([s.current  for s in self.sensors])

        def cycle(self, dT=1.0):
                cur_temp = self.readCurrentTemperature()
                self.Range = self.maxTemp - self.target_temp
                x = cur_temp - self.target_temp
                print [int(cur_temp)], x,
                # Luefterdrehzahl als lineare Gleichung y = mx + b
                # wobei gilt:
                # x = cur_temp - target_temp
                # m = (FAN_MAX - FAN_MIN) / Range 
                # b = FAN_MIN
                rpm = skal(x, 0, self.Range, self.FAN_MIN, self.FAN_MAX)
                if self.noLED == False:
                   if rpm == self.FAN_MIN:
                        open("/proc/stb/fp/led_set", "w").write("00")
                   else:
                        open("/proc/stb/fp/led_set", "w").write("05")                
                if self.noALERT == False:
                   if rpm == self.FAN_MAX:
                        if self.Alert:
                             open("/proc/stb/fp/led_set", "w").write("05")
                             self.Alert = False
                        else:
                             open("/proc/stb/fp/led_set", "w").write("80")
                             self.Alert = True
                   else:
                        self.Alert = False
                return rpm
                     
FanParameter()

if FanOK == True:
     fl = FanLoopPID(FPFanControl(FanType))
     tl = TemperatureLoop([FPTemperature(x) for x in range(8)])
     # target_temp wird begrenzt auf 40 bis 50 Grad C
     tl.target_temp = clamp (int(sys.argv[1]),40,50)
     # Grnddrehzahl wird begrenzt auf 1/10 max. Drehzahl bis max. Drehzahl - 500rpm
     if FanType == 0:
          tl.FAN_MIN = clamp (int (sys.argv[2]), minPWM, maxRPM - 500)
     else:
          tl.FAN_MIN = clamp (int (sys.argv[2]), maxRPM / 10, maxRPM - 500)
     tl.FAN_MAX = maxRPM
     if sys.argv[3] == "noLED":
          tl.noLED = True
     if sys.argv[4] == "noALERT":
          tl.noALERT = True
     dT = 1
     print
     print "#################################################################"
     print "# fan_ctrl by tmbinc (DMM) - modified by pollywolly (Vip-Board) #"
     print "# v 0.3.1 - (Luefter dreht bestaendig mit Basisdrehzahl)        #"
     print "# ###############################################################"
     print "#"
     print "# Lueftermodell: ",FanString [FanType]
     print "# ---------------------------------------------------------------"
     print "# Basisdrehzahl: ", tl.FAN_MIN, "rpm  ", "max. Drehzahl: ", tl.FAN_MAX, "rpm"
     print "# ---------------------------------------------------------------"
     print "# Solltemperatur: ", tl.target_temp, "Grad C  Maximaldrehzahl bei: ", tl.maxTemp, "Grad C"
     print "#"   
     print "#################################################################"
     print
          
     while True:
             fl.target_rpm =  tl.cycle(dT)
             fl.cycle(dT)                                                         
             time.sleep(dT)
             
else:
     print
     print "no fan detected - leaving fan_ctrl"
